function ___$insertStyle(css) {
  if (!css) {
    return;
  }
  if (typeof window === 'undefined') {
    return;
  }

  var style = document.createElement('style');

  style.setAttribute('type', 'text/css');
  style.innerHTML = css;
  document.head.appendChild(style);
  return css;
}

import { jsx } from '@emotion/core';
import { Sampler, Frequency } from 'tone';
import { useState, useEffect, useRef, useLayoutEffect, Fragment } from 'react';
import range from 'lodash.range';
import { midi, get, fromMidiSharps } from '@tonaljs/note';
import classNames from 'classnames';

function set(array, pos, item) {
    var newArray = array.slice(0);
    newArray[pos] = item;
    return newArray;
}

var makeTuning = function (text) {
    return text
        .split(' ')
        .map(function (name) { var _a; return (_a = midi(name)) !== null && _a !== void 0 ? _a : 0; })
        .reverse();
};
var tunings = {
    standard: makeTuning('E2 A2 D3 G3 B3 E4'),
    rondeña: makeTuning('D2 A2 D3 F#3 B3 E4'),
    dadgad: makeTuning('D2 A2 D3 G3 A3 D4'),
    ukelele: makeTuning('G4 C4 E4 A4'),
    lute: makeTuning('E2 A2 D3 F#3 B3 E4')
};

function useSound(samples, fretting, tuning, muted) {
    if (tuning === void 0) { tuning = tunings.standard; }
    var _a = useState(false), loaded = _a[0], setLoaded = _a[1];
    var _b = useState(), synth = _b[0], setSynth = _b[1];
    var _c = useState(tuning.map(function () { return false; })), playing = _c[0], setPlaying = _c[1];
    useEffect(function () {
        if (!muted) {
            var synth_1 = new Sampler(samples, function () { return setLoaded(true); }).toMaster();
            setSynth(synth_1);
            return function () {
                synth_1.dispose();
            };
        }
    }, [muted]);
    var play = function (string, when) {
        if (when === void 0) { when = '+0'; }
        if (loaded && !muted && synth && fretting[string] >= 0) {
            setPlaying(function (playing) { return set(playing, string, true); });
            setTimeout(function () { return setPlaying(function (playing) { return set(playing, string, false); }); }, 3000);
            synth.triggerAttackRelease(Frequency(tuning[string] + fretting[string], 'midi').toFrequency(), 4, when);
        }
    };
    var strum = function (up) {
        range(tuning.length).forEach(function (i) {
            var string = !up ? tuning.length - i - 1 : i;
            play(string, "+" + 0.05 * i);
        });
    };
    return { play: play, strum: strum, playing: playing };
}

var makeTheme = function (theme) { return theme; };
var spanishTheme = makeTheme({
    color: '#333333',
    nut: { color: '#fffacd' },
    fret: {
        color: '#9e6429',
        separator: { color: '#daa520' },
        counter: { color: '#606c76' }
    },
    string: { color: function () { return '#f3f3f3'; } },
    finger: { color: '#606c76' }
});

___$insertStyle(".frets {\n  position: relative;\n  white-space: nowrap;\n  overflow-y: auto;\n}\n.frets > li {\n  width: 10em;\n  display: inline-flex;\n  flex-direction: row-reverse;\n  position: relative;\n  border-top-width: 0.5em;\n  border-bottom-width: 2em;\n  border-top-style: solid;\n  border-bottom-style: solid;\n  vertical-align: top;\n}\n.frets > li.nut {\n  width: 7em;\n  flex-shrink: 0;\n  z-index: 1;\n}\n.frets > li:not(.nut)::before {\n  content: \"\";\n  position: absolute;\n  top: 0;\n  bottom: 0;\n  width: 0.3em;\n  display: inline-block;\n}\n.frets > li .marker {\n  position: absolute;\n  left: 0;\n  right: 0;\n  top: 0;\n  bottom: 0;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n}\n.lefty .frets > li .marker {\n  transform: scale(-1, 1);\n}\n.frets > li .counter {\n  width: 100%;\n  height: 6%;\n  position: absolute;\n  bottom: -1.5em;\n  font-weight: bold;\n  text-align: center;\n}\n\n.strings {\n  width: 100%;\n  display: flex;\n  flex-direction: column;\n  height: 20em;\n}\n.strings > li {\n  z-index: 1;\n  margin: 0;\n  position: relative;\n  display: flex;\n  flex-grow: 1;\n  align-items: center;\n  justify-content: center;\n}\n.strings > li::after {\n  content: \"\";\n  width: 100%;\n  height: 0.6em;\n  position: absolute;\n  left: 0;\n  border-bottom: solid 0.1em #555;\n}\n.strings > li label {\n  font-size: 1em;\n  position: absolute;\n  top: 0;\n  bottom: 0;\n  left: 0;\n  right: 0;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n  z-index: 2;\n  margin: 0;\n}\n.strings > li:hover input:not(:disabled) + .finger,\n.strings > li input:focus:not(:disabled) + .finger {\n  opacity: 0.5;\n}\n.strings > li input:not(:disabled) {\n  height: 100%;\n  width: 100%;\n}\n.strings > li input:not(:disabled),\n.strings > li input:not(:disabled) + .finger {\n  cursor: pointer;\n}\n.strings > li input {\n  position: absolute;\n  opacity: 0;\n}\n.strings > li input:checked + .finger {\n  opacity: 1 !important;\n}\n.strings > li input:focus:not(:disabled) + .finger {\n  box-shadow: 0 0 0 0.2em blue;\n}\n.strings > li .finger {\n  transition: opacity ease-in-out 0.1s;\n  background: white;\n  width: 5em;\n  padding: 0;\n  height: 2.3333333333em;\n  border-radius: 100px;\n  box-shadow: 0 3px 6px rgba(0, 0, 0, 0.16), 0 3px 6px rgba(0, 0, 0, 0.23);\n  line-height: 2.3333333333em;\n  text-align: center;\n  font-weight: bold;\n  opacity: 0;\n}\n\n.guitar {\n  font-family: \"Roboto\", \"Helvetica Neue\", \"Helvetica\", \"Arial\", sans-serif;\n  position: relative;\n  box-sizing: border-box;\n  display: inline-block;\n  max-width: 100%;\n  display: inline-flex;\n}\n.guitar.lefty {\n  direction: rtl;\n}\n.guitar * {\n  box-sizing: border-box;\n}\n.guitar ol,\n.guitar li {\n  margin: 0;\n  padding: 0;\n  list-style: none;\n}");

/** @jsx jsx */
function getRenderFingerSpn(tuning) {
    return function (string, fret) {
        var _a = get(fromMidiSharps(tuning[string] + fret)), letter = _a.letter, acc = _a.acc, oct = _a.oct;
        return (jsx("span", null,
            letter,
            acc === '#' ? '♯' : acc === 'b' ? '♭' : '',
            jsx("sub", null, oct)));
    };
}
function getRenderFingerRelative(tuning, root) {
    var mod = function (n, m) { return (m + (n % m)) % m; };
    return function (string, fret) { return (jsx(Fragment, null, ['1', '2m', '2', '3m', '3', '4', '5dim', '5', '5aug', '6', '7m', '7'][mod(tuning[string] + fret - root, 12)])); };
}
function Guitar(props) {
    var _a = props.strings, strings = _a === void 0 ? [] : _a, _b = props.frets, frets = _b === void 0 ? { from: 0, amount: 22 } : _b, _c = props.lefty, lefty = _c === void 0 ? false : _c, _d = props.center, center = _d === void 0 ? false : _d, renderFinger = props.renderFinger, _e = props.theme, theme = _e === void 0 ? spanishTheme : _e;
    var fretsNodeRef = useRef(null);
    var fretNodesRef = useRef({});
    useLayoutEffect(function () {
        var fretsNode = fretsNodeRef.current;
        if (center && fretsNode) {
            var pressedFrets = strings.filter(function (fret) { return fret > 0; });
            var minFret = Math.min.apply(Math, pressedFrets);
            var maxFret = Math.max.apply(Math, pressedFrets);
            var toFret = minFret + Math.floor((maxFret - minFret) / 2);
            var fretNode = fretNodesRef.current[toFret];
            if (fretNode) {
                fretsNode.scrollLeft =
                    fretNode.offsetLeft -
                        fretsNode.offsetWidth / 2 +
                        fretNode.offsetWidth / 2;
            }
        }
    }, [fretsNodeRef, fretNodesRef, strings, center, lefty]);
    return (jsx("div", { className: classNames('guitar', { lefty: lefty }, props.className) },
        jsx("ol", { className: "frets", ref: fretsNodeRef }, range(frets.from, frets.from + frets.amount + 1).map(function (fret) { return (jsx("li", { className: fret === 0 ? 'nut' : undefined, key: fret, css: {
                backgroundColor: fret === 0 ? theme.nut.color : theme.fret.color,
                borderColor: theme.color,
                '&:before': {
                    backgroundColor: theme.fret.separator.color
                }
            }, ref: function (node) { return (fretNodesRef.current[fret] = node); } },
            theme.fret.marker && (jsx("div", { className: "marker" }, theme.fret.marker(fret))),
            jsx("ol", { className: "strings" }, strings.map(function (currentFret, string) { return (jsx("li", { key: string, css: {
                    '&:after': {
                        backgroundColor: theme.string.color(string)
                    }
                }, onMouseEnter: function () { var _a; return currentFret >= 0 && ((_a = props.onPlay) === null || _a === void 0 ? void 0 : _a.call(props, string)); } },
                jsx("label", null,
                    jsx("input", { disabled: !props.onChange, type: "checkbox", checked: currentFret === fret, onChange: function () { var _a; return (_a = props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, set(strings, string, fret === 0 && strings[string] === 0
                            ? -1
                            : strings[string] === fret
                                ? 0
                                : fret)); } }),
                    jsx("span", { className: "finger", css: { color: theme.finger.color } }, renderFinger === null || renderFinger === void 0 ? void 0 : renderFinger(string, fret))))); })),
            fret !== 0 && (jsx("span", { className: "counter", css: { color: theme.fret.counter.color } }, fret)))); }))));
}

export default Guitar;
export { getRenderFingerRelative, getRenderFingerSpn, spanishTheme, tunings, useSound };
//# sourceMappingURL=index.esm.js.map
